@@ -531,7 +531,7 @@ public:
 	 * Application operator. Apply the validation algorithm
 	 * specific to multicast addresses
 	 */
-	inline void 
+	void 
 	operator()(const in6_addr address) const; 
 };
 
