@@ -47,31 +47,31 @@
 #ifndef	CCXX_MISC_H_
 #define	CCXX_MISC_H_
 
-#ifndef CCXX_CONFIG_H_
-#include <cc++/config.h>
+#ifndef CCXX_MISSING_H_
+#include <cc++/missing.h>
 #endif
 
 #ifndef CCXX_THREAD_H_
 #include <cc++/thread.h>
 #endif
 
-#include <fstream>
-#include <iostream>
-
 #define	KEYDATA_INDEX_SIZE	97
 #define	KEYDATA_PAGER_SIZE	512
-#define KEYDATA_PATH_SIZE	256
+
+#if defined(PATH_MAX)
+#if PATH_MAX > 512
+#define KEYDATA_PATH_SIZE	512
+#else
+#define	KEYDATA_PATH_SIZE	PATH_MAX
+#endif
+#else
+#define	KEYDATA_PATH_SIZE	256
+#endif
 
 #ifdef	CCXX_NAMESPACES
 namespace ost {
 #endif
 
-
-#ifdef WIN32
-class CCXX_CLASS_EXPORT MemPager;
-class CCXX_CLASS_EXPORT SharedMemPager;
-#endif
-
 /**
  * The memory pager is used to allocate cumulative memory pages for
  * storing object specific "persistant" data that is presumed to persist
@@ -87,16 +87,18 @@ class CCXX_CLASS_EXPORT SharedMemPager;
  * @author David Sugar <dyfet@ostel.com>
  * @short Accumulative object memory allocator.
  */
-class MemPager
+class __EXPORT MemPager
 {
 private:
-	unsigned int pagesize;
+	friend class String;
+
+	size_t pagesize;
 	unsigned int pages;
 
 	struct _page
 	{
 		struct _page *next;
-		int used;
+		size_t used;
 	} *page;
 
 protected:
@@ -140,7 +142,7 @@ protected:
 	 * @param str string to allocate and copy into paged memory pool.
 	 * @return copy of string from allocated memory.
 	 */
-	char* alloc(char *str);
+	char* alloc(const char *str);
 
 	/**
 	 * Create a paged memory pool for cumulative storage.  This
@@ -151,7 +153,7 @@ protected:
 	 *
 	 * @param pagesize page size to allocate chunks.
 	 */
-	MemPager(int pagesize = 4096);
+	MemPager(size_t pagesize = 4096);
 
 	/**
 	 * purge the current memory pool.
@@ -182,15 +184,16 @@ public:
  * @author David Sugar <dyfet@ostel.com>
  * @short mutex protected memory pager.
  */
-class SharedMemPager : public MemPager, public Mutex
+class __EXPORT SharedMemPager : public MemPager, public Mutex
 {
 protected:
 	/**
 	 * Create a mempager mutex pool.
 	 *
 	 * @param pagesize page size for allocation.
+	 * @param name a name for the pool.
 	 */
-	SharedMemPager(int pagesize = 4096);
+	SharedMemPager(size_t pagesize = 4096, const char *name = NULL);
 
 	/**
 	 * Purge the memory pool while locked.
@@ -281,7 +284,7 @@ protected:
  * @author David Sugar <dyfet@ostel.com>
  * @short load text configuration files into keyword pairs.
  */
-class Keydata : protected MemPager
+class __EXPORT Keydata : protected MemPager
 {
 public:
 #if defined(__GNUC__) || !defined(__hpux)
@@ -316,7 +319,8 @@ public:
 private:
 	static std::ifstream *cfgFile;
 	static char lastpath[KEYDATA_PATH_SIZE + 1];
-	static int count, sequence;
+	static int count;
+	static int sequence;
 
 	int link;
 
@@ -331,7 +335,7 @@ private:
 	unsigned getIndex(const char *sym);
 
 protected:
-	CCXX_MEMBER_EXPORT(Keysym*) getSymbol(const char *sym, bool create);
+	Keysym* getSymbol(const char *sym, bool create);
 
 	/**
 	 * Load additional key values into the currrent object from
@@ -345,7 +349,7 @@ protected:
 	 * @param keypath (filepath/section)
 	 * @param environment string
 	 */ 
-	CCXX_MEMBER_EXPORT(void) load(const char *keypath, 
+	void load(const char *keypath, 
 		  const char *environment = "CONFIG_KEYDATA");
 
 	/**
@@ -363,7 +367,7 @@ protected:
 	 * @param environment optional environment variable that
 	 * specifies the home path.
 	 */ 
-	CCXX_MEMBER_EXPORT(void) loadPrefix(const char *prefix,
+	void loadPrefix(const char *prefix,
 		  const char *keypath, const char *environment = "CONFIG_KEYDATA");
 
 	/**
@@ -374,13 +378,13 @@ protected:
 	 *
 	 * @param pairs list of NULL terminated default keyword/value pairs.
 	 */
-	CCXX_MEMBER_EXPORT(void) load(Define *pairs);
+	void load(Define *pairs);
 	
 public:
 	/**
 	 * Create an empty key data object.
 	 */
-	CCXX_MEMBER_EXPORT(CCXX_EMPTY) Keydata();
+	Keydata();
 
 	/**
 	 * Create a new key data object and use "Load" method to load an
@@ -390,14 +394,14 @@ public:
 	 * @param environment optional environment variable that
 	 * specifies the home path.
 	 */
-	CCXX_MEMBER_EXPORT(CCXX_EMPTY) Keydata(const char *keypath, const char *environment="CONFIG_KEYDATA");
+	Keydata(const char *keypath, const char *environment="CONFIG_KEYDATA");
 
 	/**
 	 * Destroy the keydata object and all allocated memory.  This
 	 * may also clear the "cache" file stream if no other keydata
 	 * objects currently reference it.
 	 */
-	CCXX_MEMBER_EXPORT(virtual) ~Keydata();
+	virtual ~Keydata();
 
 	/**
 	 * Unlink the keydata object from the cache file stream.  This
@@ -406,7 +410,7 @@ public:
 	 * the cfgFile stream will remain open.  You can also use
 	 * endKeydata().
 	 */
-	CCXX_MEMBER_EXPORT(void) unlink(void);
+	void unlink(void);
 
 	/**
 	 * Get a count of the number of data "values" that is associated
@@ -416,7 +420,7 @@ public:
 	 * @param sym keyword symbol name.
 	 * @return count of values associated with keyword.
 	 */
-	CCXX_MEMBER_EXPORT(int) getCount(const char *sym);
+	int getCount(const char *sym);
 
 	/**
 	 * Get the first data value for a given keyword.  This will
@@ -425,7 +429,7 @@ public:
 	 * @param sym keyword symbol name.
 	 * @return first set value for this symbol.
 	 */
-	CCXX_MEMBER_EXPORT(const char*) getFirst(const char *sym);
+	const char* getFirst(const char *sym);
 
 	/**
 	 * Get the last (most recently set) value for a given keyword.
@@ -434,7 +438,7 @@ public:
 	 * @param sym keywork symbol name.
 	 * @return last set value for this symbol.
 	 */
-	CCXX_MEMBER_EXPORT(const char*) getLast(const char *sym);
+	const char* getLast(const char *sym);
 
 	/**
 	 * Get an index array of ALL keywords that are stored by the
@@ -444,7 +448,7 @@ public:
 	 * @param data pointer of array to hold keyword strings.
 	 * @param max number of entries the array can hold.
 	 */
-	CCXX_MEMBER_EXPORT(unsigned) getIndex(char **data, unsigned max);
+	unsigned getIndex(char **data, unsigned max);
 
 	/**
 	 * Get the count of keyword indexes that are actually available
@@ -452,7 +456,7 @@ public:
 	 *
 	 * @return number of keywords found.
 	 */
-	CCXX_MEMBER_EXPORT(unsigned) getCount(void);
+	unsigned getCount(void);
 
 	/**
 	 * Set (replace) the value of a given keyword.  This new value
@@ -462,7 +466,7 @@ public:
 	 * @param sym keyword name to set.
 	 * @param data string to store for the keyword.
 	 */
-	CCXX_MEMBER_EXPORT(void) setValue(const char *sym, const char *data);
+	void setValue(const char *sym, const char *data);
 
 	/**
 	 * Return a list of all values set for the given keyword
@@ -471,7 +475,7 @@ public:
 	 * @return list pointer of array holding all keyword values.
 	 * @param sym keyword name to fetch.
 	 */
-	CCXX_MEMBER_EXPORT(const char * const*) getList(const char *sym);
+	const char * const* getList(const char *sym);
 
 	/**
 	 * Clear all values associated with a given keyword.  This does
@@ -479,7 +483,7 @@ public:
 	 *
 	 * @return keyword name to clear.
 	 */
-	CCXX_MEMBER_EXPORT(void) clrValue(const char *sym);
+	void clrValue(const char *sym);
 
 	/**
 	 * A convient notation for accessing the keydata as an associative
@@ -488,11 +492,17 @@ public:
 	inline const char *operator[](const char *keyword)
 		{return getLast(keyword);};
 
+	/**
+	 * static member to end keydata i/o allocations.
+	 */
+	static void end(void);
+
 	/**
 	 * Shutdown the file stream cache.  This should be used before
 	 * detaching a deamon, <code>exec()</code>, <code>fork()</code>, etc.
 	 */
-	friend CCXX_EXPORT(void) endKeydata(void);	
+	friend inline void endKeydata(void)
+		{Keydata::end();};
 };
 
 /**
@@ -519,7 +529,7 @@ public:
  *
  * The iterator has one special method 'nextDelimiter()' which returns
  * a character containing the next delimiter following this
- * tokenization process or '\0', if there are no following delimiters. In
+ * tokenization process or '\\0', if there are no following delimiters. In
  * case of skipAllDelim, it returns the FIRST delimiter.
  *
  * With the method 'setDelimiters(const char*)' you may change the
@@ -538,7 +548,7 @@ public:
  * @author Henner Zeller <H.Zeller@acm.org>
  * @license LGPL
  */
-class CCXX_CLASS_EXPORT StringTokenizer {
+class __EXPORT StringTokenizer {
 public:
 	/**
 	 * a delimiter string containing all usual whitespace delimiters.
@@ -563,7 +573,7 @@ public:
 	 * The input forward iterator for tokens.
 	 * @author Henner Zeller
 	 */
-	class CCXX_CLASS_EXPORT iterator {
+	class __EXPORT iterator {
 		friend class StringTokenizer;  // access our private constructors
 	private:
 		const StringTokenizer *myTok; // my StringTokenizer
@@ -625,7 +635,7 @@ public:
 		
 		/**
 		 * returns the next delimiter after the current token or
-		 * '\0', if there are no following delimiters.
+		 * '\\0', if there are no following delimiters.
 		 * It returns the very next delimiter (even if 
 		 * skipAllDelim=true).
 		 */
@@ -744,7 +754,7 @@ public:
 };
 
 #ifdef	CCXX_NAMESPACES
-};
+}
 #endif
 
 #endif
