@@ -46,8 +46,8 @@
 #ifndef	CCXX_UNIX_H_
 #define	CCXX_UNIX_H_
 
-#ifndef CCXX_CONFIG_H_
-#include <cc++/config.h>
+#ifndef CCXX_MISSING_H_
+#include <cc++/missing.h>
 #endif
 
 #ifndef	CCXX_SOCKET_H_
@@ -397,7 +397,7 @@ namespace ost {
 #endif // ndef WIN32
 
 #ifdef	CCXX_NAMESPACES
-};
+}
 #endif
 
 #endif
