@@ -44,6 +44,9 @@
 //
 
 #include <cc++/config.h>
+#include <cc++/string.h>
+#include <cc++/thread.h>
+#include <cc++/exception.h>
 #include <cc++/export.h>
 #include <cc++/cmdoptns.h>
 
@@ -63,9 +66,7 @@
 #include <cstdlib>
 #include <iostream>
 #include <fstream>
-#include <string>
 
-using std::string;
 using std::fstream;
 
 #ifdef	CCXX_NAMESPACES
@@ -305,11 +306,11 @@ public:
 	char		* fail_arg;
 	bool		usage_string_set;
 	bool		required_errors_set;
-	string		error_msgs;
+	String		error_msgs;
 	CommandOption	* fail_option;
 	CommandOption	* trailing;
 
-	string			  usage_string;
+	String			  usage_string;
 
 	virtual ~CommandOptionParse_impl()
 	{
@@ -492,12 +493,12 @@ public:
 		// Now check to see that all required args made it !
 
 		for ( int i = 0; i < num_options; i ++ ) {
-			CommandOption	* to = opt_list[ i ];
+			CommandOption	* toq = opt_list[ i ];
 
 			// Tell this parameter that it's done now.
-			to->parseDone( this );
+			toq->parseDone( this );
 			
-			if ( to->required && ! to->hasValue() ) {
+			if ( toq->required && ! toq->hasValue() ) {
 				has_err = true;
 				break;
 			}
@@ -518,7 +519,7 @@ public:
 		if ( fail_arg ) {
 			error_msgs = error_msgs + "Unknown/malformed option '" + fail_arg + "' \n";
 		} else if ( fail_option ) {
-			string name;
+			String name;
 			bool name_msg;
 			if ( fail_option->optionName ) {
 				name_msg = true;
@@ -566,13 +567,13 @@ public:
 	{
 		if ( usage_string_set ) return;
 
-		string	str( "" );
+		String	str( "" );
 		
-		string	str_argv0 = argv[ 0 ];
+		String	str_argv0 = argv[ 0 ];
 
 		str = str + "Usage : ";
 
-		string::size_type slashpos = str_argv0.rfind('/');
+		String::size_type slashpos = str_argv0.rfind('/');
 		if ( slashpos > str_argv0.length() ) {
 			slashpos = 0;
 		} else {
@@ -663,7 +664,7 @@ const char * CommandOptionParse_impl::printErrors()
 }
 
 #ifdef	CCXX_NAMESPACES
-};
+}
 #endif
 
 /** EMACS **
