@@ -40,8 +40,9 @@
 
 #include <cc++/config.h>
 #include <cc++/export.h>
-#include <cc++/misc.h>
 #include <cc++/slog.h>
+#include <cc++/thread.h>
+#include <cc++/misc.h>
 #include "private.h"
 
 #ifdef	CCXX_NAMESPACES
@@ -49,18 +50,42 @@ namespace ost {
 using std::endl;
 #endif
 
-MemPager::MemPager(int pg)
+const static size_t palign = __BYTE_ALIGNMENT;
+
+#if defined(HAVE_POSIX_MEMALIGN) && __BYTE_ALIGNMENT < 2
+#undef	HAVE_POSIX_MEMALIGN
+#endif
+
+#ifdef	HAVE_POSIX_MEMALIGN
+static volatile size_t malign = 0;
+#endif
+
+MemPager::MemPager(size_t pg)
 {
-	pagesize = pg;
+	pagesize = ((pg + sizeof(void *) - 1) / sizeof(void *)) * sizeof(void *);
 	pages = 1;
-	page = (struct _page *)::new char[pagesize];
-	page->next = NULL;
-	page->used = sizeof(struct _page);
+
 #ifdef	COMMON_MEMORY_AUDIT
 	slog(Slog::levelDebug) << "MemPager: creating pool, id=" << this << endl;
 	slog(Slog::levelDebug) << "MemPager: alloc, id="
 		<< this << " page=" << page << endl;
 #endif
+#if defined(HAVE_POSIX_MEMALIGN)
+	unsigned p2 = 2;
+	if(!malign)
+	{
+		while(p2 < sizeof(void *))
+			p2 *= 2;
+		while(((p2 / sizeof(void *)) * sizeof(void *)) != p2)
+			p2 *= 2;
+		malign = p2;
+	}
+	posix_memalign((void **)&page, malign, pagesize);
+#else
+	page = (struct _page *)::new void *[pagesize / sizeof(void *)];
+#endif
+        page->next = NULL;
+        page->used = sizeof(struct _page);
 }
 
 MemPager::~MemPager()
@@ -74,7 +99,11 @@ MemPager::~MemPager()
 		slog(Slog::levelDebug) << "MemPager: delete, id="
 			<< this << " page=" << root << endl;
 #endif
-		::delete[] root;
+#ifdef	HAVE_POSIX_MEMALIGN
+		::free(root);
+#else
+		delete[] root;
+#endif
 		root = page;
 	}
 #ifdef	COMMON_MEMORY_AUDIT
@@ -93,7 +122,11 @@ void MemPager::purge(void)
 		slog(Slog::levelDebug) << "Mempager: delete, id="
 			<< this << " page=" << root << endl;
 #endif
-		::delete[] root;
+#ifdef	HAVE_POSIX_MEMALIGN
+		::free(root);
+#else
+		delete[] root;
+#endif
 		--pages;
 		root = page;
 	}
@@ -105,9 +138,21 @@ void *MemPager::alloc(size_t size)
 {
 	char *ptr;
 	struct _page *npage;
+
+	if(palign > 1)
+	{
+		size_t align = size % palign;
+		if (align) size += palign - align;
+	}
+
 	if(page->used + size > pagesize)
 	{
-		npage = (struct _page *) ::new char[pagesize];
+#if defined(HAVE_POSIX_MEMALIGN)
+		posix_memalign((void **)&npage, malign, pagesize);
+#else
+		npage = (struct _page *) ::new void *[pagesize / sizeof(void *)];
+#endif
+
 #ifdef	COMMON_MEMORY_AUDIT
 		slog(Slog::levelDebug) << "MemPager: alloc, id="
 			<< this << " page=" << npage << endl;
@@ -128,6 +173,12 @@ void *MemPager::first(size_t size)
 	struct _page *npage = page;
 	char *ptr;
 
+	if(palign > 1)
+	{
+		size_t align = size % palign;
+		if (align) size += palign - align;
+	}
+
 	while(npage)
 	{
 		if(npage->used + size <= pagesize)
@@ -144,23 +195,23 @@ void *MemPager::first(size_t size)
 	return (void *)ptr;
 }
 
-char *MemPager::alloc(char *str)
+char *MemPager::alloc(const char *str)
 {
-	char *cp = (char *)alloc(strlen(str) + 1);
-	strcpy(cp, str);
-	return cp;
+	size_t len = strlen(str) + 1;
+	char *cp = (char *)alloc(len);
+	return setString(cp, len, str);
 }
 
 char *MemPager::first(char *str)
 {
-	char *cp = (char *)first(strlen(str) + 1);
-	strcpy(cp, str);
-	return cp;
+	size_t len = strlen(str) + 1;
+	char *cp = (char *)first(len);
+	return setString(cp, len, str);
 }
 
-SharedMemPager::SharedMemPager(int pg) :
-MemPager(pg), Mutex()
-{};
+SharedMemPager::SharedMemPager(size_t pg, const char *name) :
+MemPager(pg), Mutex(name)
+{}
 
 void SharedMemPager::purge(void)
 {
@@ -190,7 +241,7 @@ void *SharedMemPager::alloc(size_t size)
 }
 
 #ifdef	CCXX_NAMESPACES
-};	
+}
 #endif
 
 /** EMACS **
