@@ -40,10 +40,12 @@
 
 #include <cc++/config.h>
 #include <cc++/export.h>
+#include <cc++/exception.h>
 #include <cc++/thread.h>
 #include <cc++/address.h>
 #include <cc++/socket.h>
 #include "private.h"
+#include "nat.h"
 #include <fcntl.h>
 #include <cerrno>
 #include <cstdlib>
@@ -899,6 +901,60 @@ IPV6Host Socket::getIPV6Local(tpport_t *port) const
 
 #endif
 
+IPV4Host Socket::getIPV4NAT(tpport_t *port) const
+{
+	struct sockaddr_in addr;
+	socklen_t len = sizeof(addr);
+	natResult res;
+
+	if((res=natv4Lookup(so, &addr))!=natOK)
+	{
+	  	if(res==natNotSupported)
+		  	error( errServiceUnavailable, natErrorString( res ) );
+		else if(res==natSearchErr)
+		  	error( errSearchErr, natErrorString( res ) );
+		else
+		  	error( errLookupFail, natErrorString( res ), socket_errno );
+		if(port)
+			*port = 0;
+		memset(&addr.sin_addr, 0, sizeof(addr.sin_addr));
+	}
+	else
+	{
+		if(port)
+	    		*port = ntohs(addr.sin_port);
+	}
+	return IPV4Host(addr.sin_addr);
+}
+
+#ifdef  CCXX_IPV6
+IPV6Host Socket::getIPV6NAT(tpport_t *port) const
+{
+	struct sockaddr_in6 addr;
+	socklen_t len = sizeof(addr);
+	natResult res;
+
+	if((res=natv6Lookup(so, &addr))!=natOK)
+	{
+	  	if(res==natNotSupported)
+		  	error( errServiceUnavailable, natErrorString( res ) );
+		else if(res==natSearchErr)
+		  	error( errSearchErr, natErrorString( res ) );
+		else
+		  	error( errLookupFail, natErrorString( res ), socket_errno );
+		if(port)
+			*port = 0;
+		memset(&addr.sin6_addr, 0, sizeof(addr.sin6_addr));
+	}
+	else
+	{
+		if(port)
+	    		*port = ntohs(addr.sin6_port);
+	}
+	return IPV6Host(addr.sin6_addr);
+}
+#endif
+
 IPV4Host Socket::getIPV4Peer(tpport_t *port) const
 {
 	struct sockaddr_in addr;
@@ -1714,8 +1770,10 @@ iostream((streambuf *)this)
 void TCPStream::endStream(void)
 {
 	if(bufsize) sync();
-	delete[] gbuf;
-	delete[] pbuf;
+	if(gbuf)
+		delete[] gbuf;
+	if(pbuf)
+		delete[] pbuf;
 	gbuf = pbuf = NULL;
 	bufsize = 0;
 	clear();
