@@ -41,6 +41,12 @@
 #include <cc++/config.h>
 #include <cc++/export.h>
 #include <cc++/thread.h>
+#include <cc++/exception.h>
+#ifdef	__BORLANDC__
+#include <stdio.h>
+#else
+#include <cstdio>
+#endif
 #include "private.h"
 
 #ifdef CCXX_HAVE_NEW_INIT
@@ -92,7 +98,7 @@ static void ccxx_sig_handler(int signo);
 
 #ifdef	CCXX_SIG_THREAD_CANCEL
 
-extern "C" static RETSIGTYPE sigcancel(int)
+extern "C" RETSIGTYPE _th_sigcancel(int sig)
 {
 	pthread_exit(NULL);
 }
@@ -101,9 +107,11 @@ extern "C" static RETSIGTYPE sigcancel(int)
 
 #ifdef WIN32
 typedef	unsigned (__stdcall *exec_t)(void *);
-#if defined(_MSC_VER) && !defined(_DLL)
-# error This project cannot be compiled as a static library. Some implementation stuff require DLL
-#endif
+#ifndef CCXX_NO_DLL
+# if defined(_MSC_VER) && !defined(_DLL)
+#  error This project cannot be compiled as a static library. Some implementation stuff require DLL
+# endif
+#endif // CCXX_NO_DLL
 #endif // WIN32
 
 /*
@@ -118,8 +126,11 @@ typedef	unsigned (__stdcall *exec_t)(void *);
 #define CCXX_SUSPEND_MODE_RECURSIVE 1
 #define CCXX_SUSPEND_MODE_NOT_RECURSIVE 2
 #define CCXX_SUSPEND_MODE_ONE_SIGNAL 3
+#define	CCXX_SUSPEND_MODE_MACH 4
 
-#ifdef HAVE_PTHREAD_SUSPEND
+#if defined(_THR_MACH)
+#define	CCXX_SUSPEND_MODE CCXX_SUSPEND_MODE_MACH
+#elif defined(HAVE_PTHREAD_SUSPEND)
 #define CCXX_SUSPEND_MODE CCXX_SUSPEND_MODE_NOT_RECURSIVE
 static inline void ccxx_resume(cctid_t tid) { pthread_continue(tid); }
 static inline void ccxx_suspend(cctid_t tid) { pthread_suspend(tid); }
@@ -160,6 +171,43 @@ static inline void ccxx_suspend(cctid_t tid) { pthread_suspend(tid); }
 #endif
 #endif // ndef WIN32
 
+Thread::Cancel Thread::enterCancel(void)
+{
+	Thread *th = getThread();
+
+	if(!th)
+		return cancelInitial;
+
+	Cancel old = th->_cancel;	
+	if(old != cancelDisabled && old != cancelImmediate)
+	{
+		th->setCancel(cancelImmediate);
+#ifdef	WIN32
+		Thread::yield();
+#else
+		pthread_testcancel();
+#endif
+	}
+
+	return old;
+}
+
+void Thread::exitCancel(Cancel old)
+{
+	Thread *th = getThread();
+
+	if(!th)
+		return;
+
+	if(old != th->_cancel)
+	{
+#ifndef	WIN32
+		pthread_testcancel();
+#endif
+		th->setCancel(old);
+	}
+}
+
 void Thread::suspend(void)
 {
 	if(!priv)
@@ -168,8 +216,10 @@ void Thread::suspend(void)
 #ifdef WIN32
 	if (!priv->_active || !priv->_suspendEnable)
 	{
-		if (getException() != throwNothing)
+#ifdef	CCXX_EXCEPTIONS
+		if (Thread::getException() != throwNothing)
 			throw this;
+#endif
 		return;
 	}
 	SuspendThread(priv->_hThread);
@@ -177,6 +227,9 @@ void Thread::suspend(void)
 #else
 
 	if (!priv->_suspendEnable) return;
+#if CCXX_SUSPEND_MODE == CCXX_SUSPEND_MODE_MACH
+	thread_suspend(priv->_mach);
+#endif
 #if CCXX_SUSPEND_MODE == CCXX_SUSPEND_MODE_RECURSIVE
 	ccxx_suspend(priv->_tid);
 #endif
@@ -189,6 +242,20 @@ void Thread::suspend(void)
 #endif // WIN32
 }
 
+#if defined(__FreeBSD__)
+#define	AUTOSTACK	0x10000
+#endif
+
+#if defined(MACOSX)
+#define	AUTOSTACK	0
+#endif
+
+#ifndef	AUTOSTACK
+#define	AUTOSTACK	0x100000
+#endif
+
+size_t Thread::_autostack = AUTOSTACK;
+
 void Thread::resume(void)
 {
 	if(!priv)
@@ -197,14 +264,19 @@ void Thread::resume(void)
 #ifdef WIN32
 	if (!priv->_active || !priv->_suspendEnable)
 	{
-		if (getException() != throwNothing)
+#ifdef	CCXX_EXCEPTIONS
+		if (Thread::getException() != throwNothing)
 			throw this;
+#endif
 		return;
 	}
 	ResumeThread(priv->_hThread);
 
 #else
 	if (!priv->_suspendEnable) return;
+#if CCXX_SUSPEND_MODE == CCXX_SUSPEND_MODE_MACH
+	thread_resume(priv->_mach);
+#endif
 #if CCXX_SUSPEND_MODE == CCXX_SUSPEND_MODE_RECURSIVE
 	ccxx_resume(priv->_tid);
 #endif
@@ -225,7 +297,32 @@ void Thread::resume(void)
 
 void Thread::join(void)
 {
+	bool detached = isDetached();
 	joinSem.wait();
+	if(detached)
+	{
+		joinSem.post();
+		return;
+	}
+
+#ifdef	WIN32		// wait for real w32 thread to cleanup
+
+	if(priv->_hThread)
+	{
+		WaitForSingleObject(priv->_hThread, INFINITE);
+		::CloseHandle(priv->_hThread);
+		priv->_hThread = NULL;
+	}		
+
+#else			// make sure we cleanup exiting thread
+	if(priv->_jtid)
+	{
+		pthread_join(priv->_jtid, NULL);
+	}
+
+	priv->_jtid = 0;
+#endif
+	joinSem.post();	// enable next waiting thread after cleanup
 }
 
 #ifndef WIN32
@@ -364,12 +461,10 @@ Mutex	PosixThread::_arm;
 #endif
 #endif
 
-void sigInstall(int);
+//void PosixThread::sigInstall(int);
 
 Thread::Thread(bool isMain):
-	_cancel(cancelDefault),
-	_start(NULL),
-	priv(new ThreadImpl(threadTypeDummy))
+_cancel(cancelDefault), _start(NULL), priv(new ThreadImpl(threadTypeDummy))
 {
 #ifdef WIN32
 	priv->_tid = GetCurrentThreadId();
@@ -382,9 +477,12 @@ Thread::Thread(bool isMain):
 	
 	if(isMain)
 	{
+		setName("main()");
 		priv->_type   = threadTypeMain;
 		_main = this;
 	}
+	else
+		setName("-dummy-");
 	_self.setKey(this);
 
 #else
@@ -401,11 +499,12 @@ Thread::Thread(bool isMain):
 		_main = this;
 		priv->_type = threadTypeMain;
 #if !defined(__CYGWIN32__) && !defined(__MINGW32__)
-		sigInstall(SIGHUP);
-		sigInstall(SIGALRM);
-		sigInstall(SIGPIPE);
-		sigInstall(SIGABRT);
+		PosixThread::sigInstall(SIGHUP);
+		PosixThread::sigInstall(SIGALRM);
+		PosixThread::sigInstall(SIGPIPE);
+		PosixThread::sigInstall(SIGABRT);
 
+		memset(&act, sizeof(act), 0);
 		act.sa_handler = (signalexec_t)&ccxx_sig_handler;
 		sigemptyset(&act.sa_mask);
 # ifdef	SA_RESTART
@@ -434,8 +533,9 @@ Thread::Thread(bool isMain):
 # endif
 
 # ifdef	CCXX_SIG_THREAD_CANCEL
+		memset(&act, sizeof(act), 0);
 		act.sa_flags = 0;
-		act.sa_handler = sigcancel;
+		act.sa_handler = _th_sigcancel;
 		sigemptyset(&act.sa_mask);
 		sigaddset(&act.sa_mask, SIGHUP);
 		sigaddset(&act.sa_mask, SIGALRM);
@@ -450,78 +550,95 @@ Thread::Thread(bool isMain):
 }	
 
 Thread::Thread(int pri, size_t stack):
-	_cancel(cancelDefault),
-	_start(NULL),
-	priv(new ThreadImpl(threadTypeNormal))
+_cancel(cancelDefault), _start(NULL), priv(new ThreadImpl(threadTypeNormal))
 {
 #ifdef WIN32
 	if(!_main)
 	{
 		_self.setKey(NULL);
 		_main = this;
+		setName("main()");
 	}
+	else
+		snprintf(_name, sizeof(_name), "%d", getId());
 
-	_parent = getThread();
+	_parent = Thread::get();
 	if(_parent)
 		priv->_throw = _parent->priv->_throw;
 	else
 		_parent = this;
-
+	
 	priv->_cancellation = CreateEvent(NULL, TRUE, FALSE, NULL);
 	if(!priv->_cancellation)
-		throw(this);
+		THROW(this);
 
-	priv->_hThread = (HANDLE)_beginthreadex(NULL, stack, (exec_t)&Execute, (void *)this, CREATE_SUSPENDED, (unsigned *)&priv->_tid);
-	if(!priv->_hThread)
-	{
-		CloseHandle(priv->_cancellation);
-		priv->_cancellation = NULL;
-		throw(this);
-	}
+	if(stack <= _autostack)
+		priv->_stack = 0;
+	else
+		priv->_stack = stack;
 
-	setCancel(cancelInitial);
+	if(pri > 2)
+		pri = 2;
+	if(pri < -2)
+		pri = -2;
 
 	switch(pri)
 	{
 	case 1:
-		pri = THREAD_PRIORITY_ABOVE_NORMAL;
+		priv->_priority = THREAD_PRIORITY_ABOVE_NORMAL;
 		break;
 	case -1:
-		pri = THREAD_PRIORITY_BELOW_NORMAL;
+		priv->_priority = THREAD_PRIORITY_BELOW_NORMAL;
 		break;
 	case 2:
-		pri = THREAD_PRIORITY_HIGHEST;
+		priv->_priority = THREAD_PRIORITY_HIGHEST;
 		break;
 	case -2:
-		pri = THREAD_PRIORITY_LOWEST;
+		priv->_priority = THREAD_PRIORITY_LOWEST;
 		break;
 	default:
-		pri = THREAD_PRIORITY_NORMAL;
+		priv->_priority = THREAD_PRIORITY_NORMAL;
 	}
 
-	SetThreadPriority(priv->_hThread, pri);
-
 #else
-//	int	rc;
+	int	salign;
 
 	pthread_attr_init(&priv->_attr);
 	pthread_attr_setdetachstate(&priv->_attr, PTHREAD_CREATE_JOINABLE);
 
 #ifdef	PTHREAD_STACK_MIN
-	if(stack && pthread_attr_setstacksize(&priv->_attr, stack <= PTHREAD_STACK_MIN ? PTHREAD_STACK_MIN : stack))
+	if(stack && stack <=  _autostack)
+		pthread_attr_setstacksize(&priv->_attr, _autostack);
+	else if(stack > _autostack)
 	{
-		switch(getException())
+		if(stack < PTHREAD_STACK_MIN)
+			stack = PTHREAD_STACK_MIN;
+		else	// align to nearest min boundry
+		{	
+			salign = stack / PTHREAD_STACK_MIN;
+			if(stack % PTHREAD_STACK_MIN)
+				++salign;
+			stack = salign * PTHREAD_STACK_MIN;
+		}
+		if(stack && pthread_attr_setstacksize(&priv->_attr, stack))
 		{
-		case throwObject:
-			throw(this);
-			return;
+#ifdef	CCXX_EXCEPTIONS
+			switch(Thread::getException())
+			{
+			case throwObject:
+				throw(this);
+				return;
 #ifdef	COMMON_STD_EXCEPTION
-		case throwException:
-			throw(ThrException("no stack space"));
+			case throwException:
+				throw(ThrException("no stack space"));
+				return;
+#endif
+			default:
+				return;
+			}
+#else
 			return;
 #endif
-		default:
-			return;
 		}
 	}
 #endif
@@ -538,7 +655,8 @@ Thread::Thread(int pri, size_t stack):
 		policy = sched_getscheduler(0);
 		if(policy < 0)
 		{
-			switch(getException())
+#ifdef	CCXX_EXCEPTIONS
+			switch(Thread::getException())
 			{
 			case throwObject:
 				throw(this);
@@ -551,6 +669,9 @@ Thread::Thread(int pri, size_t stack):
 			default:
 				return;
 			}
+#else
+			return;
+#endif
 		}
 
 		sched_getparam(0, &sched);
@@ -583,19 +704,20 @@ Thread::Thread(int pri, size_t stack):
 	_cancel = cancelInitial;
 
 #endif // WIN32
-};
+}
 
 #ifndef WIN32
 Thread::Thread(const Thread &th)
 {
 	priv = new ThreadImpl(threadTypeNormal);
 	_parent = th._parent;
-	// FIXME: correct ??
 	priv->_attr = th.priv->_attr;
 	_cancel = cancelInitial;
 	_start = NULL;
 	priv->_throw = th.priv->_throw;
 	priv->_suspendEnable = false;
+
+	setName(NULL);
 	
 //	sigset_t mask, newmask;
 //	int rc;
@@ -603,10 +725,10 @@ Thread::Thread(const Thread &th)
 //	pthread_sigmask(SIG_BLOCK, blocked_signals(&newmask), &mask);
 //	rc = pthread_create(&_tid, &_attr, exec_t(&ccxx_exec_handler), this);
 //	pthread_sigmask(SIG_SETMASK, &mask, NULL);
-//	if(rc && getException() == throwObject)
+//	if(rc && Thread::getException() == throwObject)
 //		throw(this);
 //#ifdef	COMMON_STD_EXCEPTION
-//	else if(rc && getException() == throwException)
+//	else if(rc && Thread::getException() == throwException)
 //		throw(ThrException("cannot start copy"));
 //#endif
 }
@@ -614,18 +736,32 @@ Thread::Thread(const Thread &th)
 
 Thread::~Thread()
 {
-//	was for 1.1.x where _msgbuf was dynamic
-//	if(priv)
-//		if( priv->_msgbuf != NULL)
-//			free(priv->_msgbuf);
-#ifdef WIN32
+	if(!priv)
+		return;
+
+#ifndef	WIN32
+	if(this == &_mainthread)
+		return;
+#endif
+
+	if(priv->_type == threadTypeDummy)
+	{
+		delete priv;
+		priv = NULL;
+		return;
+	}
+
 	terminate();
-#else
-	if(this != &_mainthread)
-		terminate();
-#endif // WIN32
 }
 
+void Thread::setName(const char *text)
+{
+	if(text)
+		snprintf(_name, sizeof(_name), "%s", text);
+	else
+		snprintf(_name, sizeof(_name), "%ld", (long)getId());
+}	
+
 bool Thread::isThread(void)
 {
 	if(!priv)
@@ -644,7 +780,8 @@ bool Thread::isDetached(void)
                 return false;
 
 #ifdef  WIN32
-        return false;   // win32 doesn't support detached threads directly
+	// win32 doesn't support detached threads directly
+	return priv->_detached;
 #else
         int state;
 
@@ -655,6 +792,11 @@ bool Thread::isDetached(void)
 #endif
 }
 
+cctid_t Thread::getId(void) const
+{
+	return priv->_tid;
+}
+
 bool Thread::isRunning(void)
 {
 	if(!priv)
@@ -666,18 +808,33 @@ bool Thread::isRunning(void)
 #endif // WIN32
 }
 
-int Thread::start(Semaphore *start)
+int Thread::start(Semaphore *st)
 {
 	if(!priv)
 		return -1;
 
 #ifdef WIN32
-	_start = start;
-	if (!priv->_active)
-	{
-		priv->_active = true;
-		ResumeThread(priv->_hThread);
-	}
+	if(priv->_active)
+		return -1;
+
+	_start = st;
+
+	priv->_hThread = (HANDLE)_beginthreadex(NULL, (unsigned)priv->_stack, (exec_t)&Execute, (void *)this, CREATE_SUSPENDED, (unsigned *)&priv->_tid);
+	if(!priv->_hThread)
+ 	{
+		CloseHandle(priv->_cancellation);
+		priv->_cancellation = NULL;
+		return -1;
+ 	}
+
+	setCancel(cancelInitial);
+
+	SetThreadPriority(priv->_hThread, priv->_priority);
+
+	ResumeThread(priv->_hThread);
+
+	priv->_active = true;
+
 	return 0;
 
 #else
@@ -692,18 +849,23 @@ int Thread::start(Semaphore *start)
 			return -1;
 	}
 
-	_start = start;
+	_start = st;
 	return pthread_create(&priv->_tid, &priv->_attr, exec_t(&ccxx_exec_handler), this);
 #endif
 }
 
-int Thread::detach(Semaphore *start)
+int Thread::detach(Semaphore *st)
 {
 	_parent = NULL;
 #ifdef WIN32
-	// win32 do not support detach	
-	if(start)
-		start->post();
+	// win32 we emulate detach	
+	if(!priv)
+		return -1;
+	priv->_detached = true;
+	if(!priv->_active)
+		return Thread::start(st);
+	else if(_start)
+		_start->post();
 	return 0;
 #else
 	int rtn;
@@ -724,7 +886,7 @@ int Thread::detach(Semaphore *start)
 
 	pthread_attr_setdetachstate(&priv->_attr, PTHREAD_CREATE_DETACHED);
 
-	_start = start;
+	_start = st;
 	rtn = pthread_create(&priv->_tid, &priv->_attr, exec_t(&ccxx_exec_handler), this);
 	if(!rtn && priv->_tid)
 		return 0;
@@ -735,82 +897,107 @@ int Thread::detach(Semaphore *start)
 void Thread::terminate(void)
 {
 #ifdef WIN32
+	HANDLE hThread;
+
 	if(!priv)
 		return;
 
+	hThread = priv->_hThread;
+
 	if (!priv->_tid || isThread())
-		return;
+	{
+		if( priv->_cancellation)
+			::CloseHandle(priv->_cancellation);
+      		if(hThread)
+            		::CloseHandle(hThread);
+		delete priv;
+		priv = NULL;
+      		return;
+	}
 
 	bool terminated = false;
-	if(!priv->_active)
+	if(!priv->_active && hThread != NULL)
 	{
 		// NOTE: add a test in testthread for multiple
 		// suspended Terminate
-		ResumeThread(priv->_hThread);
-		TerminateThread(priv->_hThread, 0);
+		ResumeThread(hThread);
+		TerminateThread(hThread, 0);
 		terminated = true;
 	}
-	else
+	else if(hThread != NULL)
 	{
 		switch(_cancel)	{
 		case cancelImmediate:
-			TerminateThread(priv->_hThread, 0);
+			TerminateThread(hThread, 0);
 			terminated = true;
 			break;
 		default:
 			SetEvent(priv->_cancellation);
 		}
 	}
-	WaitForSingleObject(priv->_hThread, INFINITE);
+	if(hThread != NULL)
+	{
+		WaitForSingleObject(hThread, INFINITE);
+		CloseHandle(hThread);
+		hThread = NULL;
+	}
 	if(_parent)
 		_parent->notify(this);
-	// FIXME: test race condition, a thread can be restarted
-	CloseHandle(priv->_hThread);
-	priv->_hThread = NULL;
-	CloseHandle(priv->_cancellation);
+	if(priv->_cancellation != NULL)
+		CloseHandle(priv->_cancellation);
 	priv->_cancellation = NULL;
 	priv->_tid = 0;
-	_self.setKey(DUMMY_INVALID_THREAD);
-	delete priv;
-	priv = NULL;
+	if(getThread() == this)
+		_self.setKey(DUMMY_INVALID_THREAD);
 
 	if (terminated)
 		final();
 #else
 	if(!priv)
 		return;
-	
-	if(!priv->_tid)
-		return;
 
-	if(pthread_self() != priv->_tid)
+        cctid_t jtid = priv->_jtid, tid = priv->_tid;
+
+	if(jtid && (pthread_self() != jtid))
+	{
+		pthread_join(jtid, NULL);
+		priv->_jtid = 0;
+	}
+	else if((pthread_self() != tid) && tid)
 	{
 		// in suspend thread cannot be cancelled or signaled
 		// ??? rigth
 		// ccxx_resume(priv->_tid);
 		
+
 		// assure thread has ran before we try to cancel...
 		if(_start)
 			_start->post();
 
-		pthread_cancel(priv->_tid);
+		pthread_cancel(tid);
 		if(!isDetached())
-			pthread_join(priv->_tid,NULL);
+		{
+			pthread_join(tid,NULL);
+			priv->_tid = 0;
+		}
 	}
 
-        // called terminate this on current thread ??
-        if (isThread())
-        {
-                priv->_tid = 0;
-                ThreadImpl::_self.setKey(DUMMY_INVALID_THREAD);
-        }
         pthread_attr_destroy(&priv->_attr);
+#endif
 	delete priv;
 	priv = NULL;
+}
+
+void Thread::sync(void)
+{
+#ifdef	__MACH__
+	Thread::exit();
+#else
+	Thread::sleep(TIMEOUT_INF);
 #endif
 }
 
-void Thread::exit()
+void Thread::exit(void)
 {
 	if (isThread())
 	{
@@ -822,10 +1009,13 @@ void Thread::exit()
 		pthread_exit(NULL);
 #endif // WIN32
 	}
+	
 }
 		
 void Thread::close()
 {
+	bool detached = isDetached();
+
 #if !defined(CCXX_SIG_THREAD_ALARM) && !defined(__CYGWIN32__) && !defined(__MINGW32__) && !defined(WIN32)
 	if(this == PosixThread::_timer)
 		PosixThread::_arm.leaveMutex();
@@ -837,7 +1027,8 @@ void Thread::close()
 	// final can call destructor (that call Terminate)
 	final();
 
-	// test if this class have been deleted
+	// test if this class is self-exiting thread
+
 #ifdef WIN32
 	if (_self.getKey() == this)
 #else
@@ -845,9 +1036,20 @@ void Thread::close()
 #endif
 	{
 		if(priv)
-			priv->_tid = 0;
+		{
+#ifndef	WIN32
+			priv->_jtid = priv->_tid;
+#else			
+			priv->_active = false;
+#endif
+		}
 		joinSem.post();
 	}
+
+	// see if detached, and hence self deleting
+
+	if(detached)
+		delete this;
 }
 
 #ifndef WIN32
@@ -872,6 +1074,11 @@ inline void ThreadImpl::ThreadExecHandler(Thread *th)
 
 	pthread_sigmask(SIG_BLOCK, blocked_signals(&mask), NULL);
 	th->priv->_tid = pthread_self();
+#if defined(HAVE_PTHREAD_MACH_THREAD_NP)
+	th->priv->_mach = pthread_mach_thread_np(th->priv->_tid);
+#elif defined(_THR_MACH)
+	th->priv->_mach = mach_thread_self();
+#endif
 	th->setCancel(Thread::cancelInitial);
 	// using SIGUSR3 do not enable suspend by default
 	th->setSuspend(Thread::suspendEnable);
@@ -897,7 +1104,9 @@ inline void ThreadImpl::ThreadExecHandler(Thread *th)
 // delete Thread class created for no CommonC++ thread
 inline void ThreadImpl::ThreadDestructor(Thread* th)
 {
-	if (!th || th == DUMMY_INVALID_THREAD)
+	if (!th || th == DUMMY_INVALID_THREAD || !th->priv)
+		return;
+	if(!th->priv)
 		return;
 	if (th->priv->_type == threadTypeDummy)
 		delete th;
@@ -916,7 +1125,7 @@ static void ccxx_exec_handler(Thread *th)
 
 #ifdef	CCXX_SIG_THREAD_CANCEL
 
-void	Thread::setCancel(thread_cancel_t mode)
+void	Thread::setCancel(Cancel mode)
 {
 	sigset_t	mask;
 
@@ -959,12 +1168,12 @@ void	Thread::setCancel(Cancel mode)
 	switch(mode)
 	{
 	case cancelImmediate:
-		pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, &old);
 		pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, &old);
+		pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, &old);
 		break;
 	case cancelDeferred:
-		pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, &old);
 		pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, &old);
+		pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, &old);
 		break;
 	case cancelInitial:
 	case cancelDisabled:
@@ -1003,13 +1212,44 @@ void	Thread::yield(void)
 #endif
 
 #ifdef	CCXX_SIG_THREAD_CANCEL
-	if(th && th->_cancel != cancelDisabled)
+	if(th && th->_cancel != cancelDisabled && th->_cancel != cancelInitial)
 		pthread_sigmask(SIG_SETMASK, &old, NULL);
 #endif
 
 #endif // WIN32
 }
 
+void Thread::setException(Thread::Throw mode)
+{
+	Thread *thread = getThread();
+	thread->priv->_throw = mode;
+}
+
+Thread::Throw Thread::getException(void)
+{
+	Thread *thread = getThread();
+	return thread->priv->_throw;
+}
+
+Cancellation::Cancellation(Thread::Cancel cancel)
+{
+	Thread *thread = getThread();
+	if(!thread)
+		return;
+
+	prior = thread->getCancel();
+	thread->setCancel(cancel);
+}
+
+Cancellation::~Cancellation()
+{
+	Thread *thread = getThread();
+	if(!thread)
+		return;
+
+	thread->setCancel(prior);
+}
+
 bool Thread::testCancel(void)
 {
 #ifdef WIN32
@@ -1022,7 +1262,7 @@ bool Thread::testCancel(void)
 		if(WaitForSingleObject(priv->_cancellation, 0) == WAIT_OBJECT_0)
 		{
 			if (_cancel == cancelManual)
-				throw InterruptException();
+				THROW(InterruptException());
 			else
 				exit();
 		}
@@ -1060,7 +1300,7 @@ bool Thread::isCancelled()
 	return waitThread(priv->_cancellation, 0) == WAIT_OBJECT_0;
 }
 
-CCXX_EXPORT(DWORD) waitThread(HANDLE hRef, timeout_t timeout)
+DWORD Thread::waitThread(HANDLE hRef, timeout_t timeout)
 {
 	Thread *th = getThread();
 
@@ -1089,7 +1329,7 @@ void Thread::sleep(timeout_t timeout)
 		if(WaitForSingleObject(th->priv->_cancellation, timeout) == WAIT_OBJECT_0)
 		{
 			if (th->_cancel == cancelManual)
-				throw InterruptException();
+				THROW(InterruptException());
 			else
 				th->exit();
 		}
@@ -1114,7 +1354,7 @@ DWORD Thread::waitHandle(HANDLE obj, timeout_t timeout)
 		switch(stat = WaitForMultipleObjects(2, objects, false, timeout)) 	{
 		case WAIT_OBJECT_0:
 			if (_cancel == cancelManual)
-				throw InterruptException();
+				THROW(InterruptException());
 			else
 				exit();
 		case WAIT_OBJECT_0 + 1:
@@ -1145,7 +1385,7 @@ BOOL WINAPI DllMain(
 #endif // WIN32
 
 #ifndef WIN32
-Thread *getThread(void)
+Thread *Thread::get(void)
 {
 	Thread *thread;
 	
@@ -1173,7 +1413,7 @@ Thread *getThread(void)
 
 #else // WIN32
 
-CCXX_EXPORT(Thread*) getThread(void)
+Thread *Thread::get(void)
 {
 	Thread *th = (Thread *)_self.getKey();
 	if (th == DUMMY_INVALID_THREAD) return NULL;
@@ -1187,12 +1427,7 @@ unsigned __stdcall Thread::Execute(Thread *th)
 {
 	_self.setKey(th);
 	th->yield();
-	if(!th->priv->_active)
-	{
-		th->setCancel(cancelImmediate);
-		th->yield();
-		return 0;	
-	}
+
 	if(th->_start)
 	{
 		th->_start->wait();
@@ -1223,7 +1458,13 @@ inline void ThreadImpl::PosixThreadSigHandler(int signo)
 {
 	Thread	*t = getThread();
 	PosixThread *th = NULL;
+
+#ifdef	CCXX_EXCEPTIONS
 	if (t) th = dynamic_cast<PosixThread*>(t);
+#else
+        if (t) th = (PosixThread*)(t);
+#endif
+
 	if (!th) return;
 
 	switch(signo)
@@ -1273,7 +1514,7 @@ static void ccxx_sig_handler(int signo)
 PosixThread::PosixThread(int pri, size_t stack):
 	Thread(pri,stack)
 {
-	time(&_alarm);
+       SysTime::getTime(&_alarm);
 }
 
 void PosixThread::setTimer(timeout_t timer, bool periodic)
@@ -1298,7 +1539,7 @@ void PosixThread::setTimer(timeout_t timer, bool periodic)
 	_arm.enterMutex();
 	_timer = this;
 #endif
-	time(&_alarm);
+       SysTime::getTime(&_alarm);
 	sigemptyset(&sigs);
 	sigaddset(&sigs, SIGALRM);
 	pthread_sigmask(SIG_UNBLOCK, &sigs, NULL);
@@ -1323,8 +1564,7 @@ timeout_t PosixThread::getTimer(void) const
 	return (timeout_t)(itimer.it_value.tv_sec * 1000 +
 		itimer.it_value.tv_usec / 1000);
 #else
-	time_t now;
-	time(&now);
+	time_t now = SysTime::getTime();
 	return (timeout_t)(((now - _alarm) * 1000) + 500);
 #endif
 }
@@ -1355,7 +1595,7 @@ void PosixThread::endTimer(void)
 #endif
 }
 
-#ifdef HAVE_SIGWAIT
+#if defined(HAVE_SIGWAIT) || defined(HAVE_SIGWAIT2)
 void	PosixThread::waitSignal(signo_t signo)
 {
 	sigset_t	mask;
@@ -1388,7 +1628,17 @@ void	PosixThread::signalThread(Thread* th,signo_t signo)
 	pthread_kill(th->priv->_tid, signo);
 }
 
-void sigInstall(int signo)
+pthread_attr_t * PosixThread::getPthreadAttrPtr(void)
+{
+ 	return &priv->_attr;
+}
+
+pthread_t PosixThread::getPthreadId(void)
+{
+	return priv->_tid;
+}
+
+void PosixThread::sigInstall(int signo)
 {
 	struct sigaction act;
 	
@@ -1404,18 +1654,21 @@ void sigInstall(int signo)
 }
 #endif
 
-#ifdef	CCXX_USE_POLL
+#ifdef	USE_POLL
 
 Poller::Poller()
 {
 	nufds = 0;
 	ufds = NULL;
-};
+}
 
 Poller::~Poller()
 {
 	if(ufds)
+	{
 		delete[] ufds;
+		ufds = NULL;
+	}
 }
 
 pollfd *Poller::getList(int cnt)
@@ -1432,11 +1685,79 @@ pollfd *Poller::getList(int cnt)
 
 #endif
 
+Mutex SysTime::timeLock;
+
+time_t SysTime::getTime(time_t *tloc) 
+{
+    time_t ret;
+    lock();
+    time_t temp;
+#ifdef	WIN32
+    ::time(&temp);
+#else
+    std::time(&temp);
+#endif
+    memcpy(&ret, &temp, sizeof(time_t));
+    if (tloc != NULL) 
+        memcpy(tloc, &ret, sizeof(time_t));
+    unlock();
+    return ret;   
+}
+
+int SysTime::getTimeOfDay(struct timeval *tp) 
+{
+	struct timeval temp;
+    int ret(0);
+    lock();
+
+#ifdef	WIN32
+    // We could use _ftime(), but it is not available on WinCE.
+    // (WinCE also lacks time.h)
+    // Note also that the average error of _ftime is around 20 ms :)
+    time(&temp.tv_sec);
+    temp.tv_usec = (GetTickCount() % 1000) * 1000;
+	memcpy(tp, &temp, sizeof(struct timeval));
+#else
+    ret = ::gettimeofday(&temp, NULL);
+    if (ret == 0) 
+        memcpy(tp, &temp, sizeof(struct timeval));
+#endif
+
+    unlock();
+    return ret;
+}
+
+struct tm *SysTime::getLocalTime(const time_t *clock, struct tm* result) 
+{
+    lock();
+#ifdef	WIN32
+    struct tm *temp = ::localtime(clock);
+#else
+    struct tm *temp = std::localtime(clock);
+#endif
+    memcpy(result, temp, sizeof(struct tm));
+    unlock();
+	return result;
+}
+
+struct tm *SysTime::getGMTTime(const time_t *clock, struct tm* result) 
+{
+    lock();
+#ifdef	WIN32
+    struct tm *temp = ::gmtime(clock);
+#else
+    struct tm *temp = std::gmtime(clock);
+#endif
+    memcpy(result, temp, sizeof(struct tm));
+    unlock();
+	return result;
+}
+
 // C stuff
 // this function must declared as extern "C" for some compiler
 
 #ifdef CCXX_NAMESPACES
-};
+}
 #endif
  
 /** EMACS **
