@@ -94,7 +94,9 @@ TCPStream(fam)
 	timeout = to;
 	protocol = protocolHttp1_0;
 	follow = true;
+	proxyAuth = authAnonymous;
 	encoding = encodingBinary;
+ 	proxyUser = proxyPasswd = NULL;
 	auth = authAnonymous;
 	cookie = agent = pragma = referer = user = password = NULL;
 	localif = NULL;
@@ -182,7 +184,7 @@ void URLStream::setProxy(const char *host, tpport_t port)
 URLStream::Error URLStream::submit(const char *path, const char **vars,
 	       int buf)
 {
-	Error status = errInvalid;
+	Error status = errInvalid, saved;
 
 	if(!strnicmp(path, "http:", 5))
 	{
@@ -198,9 +200,10 @@ URLStream::Error URLStream::submit(const char *path, const char **vars,
 	}
 	else
 	{
+		saved = status;
 		status = getHTTPHeaders();
 		if(status == errSuccess)
-			return status;
+			return saved;
 		else if(status == errTimeout)
 		{
 			if(Socket::state != AVAILABLE)
@@ -213,7 +216,7 @@ URLStream::Error URLStream::submit(const char *path, const char **vars,
 URLStream::Error URLStream::post(const char *path, const char **vars,
 		int buf)
 {
-	Error status = errInvalid;
+	Error status = errInvalid, saved;
 
 	if(!strnicmp(path, "http:", 5))
 	{
@@ -230,9 +233,10 @@ URLStream::Error URLStream::post(const char *path, const char **vars,
 	}
 	else
 	{
+		saved = status;
 		status = getHTTPHeaders();
 		if(status == errSuccess)
-			return status;
+			return saved;
 		else if(status == errTimeout)
 		{
 			if(Socket::state != AVAILABLE)
@@ -245,7 +249,7 @@ URLStream::Error URLStream::post(const char *path, const char **vars,
 
 URLStream::Error URLStream::head(const char *path, int buf)
 {
-	Error status = errInvalid;
+	Error status = errInvalid, saved;
 
 	if(!strnicmp(path, "http:", 5))
 	{
@@ -262,9 +266,10 @@ URLStream::Error URLStream::head(const char *path, int buf)
 	}
 	else
 	{
+		saved = status;
 		status = getHTTPHeaders();
 		if(status == errSuccess)
-			return status;
+			return saved;
 		else if(status == errTimeout)
 		{
 			if(Socket::state != AVAILABLE)
@@ -294,10 +299,22 @@ URLStream &URLStream::getline(char *buffer, size_t size)
 	return *this;
 }
 
+URLStream::Error URLStream::get(int buffer)
+{
+	String path = String("http://") + m_host;
+
+	if ( m_address.operator[](0) != '/' )
+		path += "/";
+
+	path += m_address;
+
+	return get(path.c_str(), buffer);
+}
+
 URLStream::Error URLStream::get(const char *urlpath, int buf)
 {
 	const char *path = urlpath;
-	Error status = errInvalid;
+	Error status = errInvalid, saved;
 
 	urlmethod = methodFileGet;
 
@@ -349,9 +366,10 @@ URLStream::Error URLStream::get(const char *urlpath, int buf)
 	}
 	else
 	{
+		saved = status;
 		status = getHTTPHeaders();
 		if(status == errSuccess)
-			return status;
+			return saved;
 		else if(status == errTimeout)
 		{
 			if(Socket::state != AVAILABLE)
@@ -522,6 +540,9 @@ retry:
 		break;
 	}
 
+	if ( m_host.empty() )
+		m_host = hp;
+
 	str << "Host: " << hp << "\r\n";
         if(agent)
                 str << "User-Agent: " << agent << "\r\n";
@@ -543,10 +564,23 @@ retry:
 		b64Encode(reloc, reloc + 64, 128);
 		str << reloc + 64 << "\r\n";
 	case authAnonymous:
-	        break;
+		break;
 	}
 
-        str << "Connection: close\r\n";
+	switch(proxyAuth)
+ 	{
+	case authBasic:
+ 		str << "Proxy-Authorization: Basic ";
+ 		snprintf(reloc, 64, "%s:%s", proxyUser, proxyPasswd);
+ 		b64Encode(reloc, reloc + 64, 128);
+ 		str << reloc + 64 << "\r\n";
+ 		str << "Proxy-Connection: close" << "\r\n";
+ 	case authAnonymous:
+ 		break;
+ 	}
+
+
+	str << "Connection: close\r\n";
 	char **add = extraHeader();
 	if(add)
 	{
@@ -777,6 +811,18 @@ void URLStream::setAuthentication(Authentication a, const char *value)
 	}
 }
 
+void URLStream::setProxyAuthentication(Authentication a, const char *value)
+{
+	proxyAuth = a;
+	if (proxyAuth != authAnonymous)
+ 	{
+ 		if(!proxyUser)
+ 			proxyUser = "anonymous";
+ 		if(!proxyPasswd)
+ 			proxyPasswd = "";
+	}
+}
+
 void URLStream::setReferer(const char *str)
 {
 	if(!str)
